@@ -1,12 +1,13 @@
 package data
 
 import (
+	"context"
 	"strings"
 
 	"github.com/hairyhenderson/gomplate/v3/libkv"
 )
 
-func readConsul(source *Source, args ...string) (data []byte, err error) {
+func readConsul(ctx context.Context, source *Source, args ...string) (data []byte, err error) {
 	if source.kv == nil {
 		source.kv, err = libkv.NewConsul(source.URL)
 		if err != nil {
