@@ -1,6 +1,7 @@
 package data
 
 import (
+	"context"
 	"strings"
 	"testing"
 
@@ -8,15 +9,14 @@ import (
 )
 
 func TestReadStdin(t *testing.T) {
-	defer func() {
-		stdin = nil
-	}()
-	stdin = strings.NewReader("foo")
-	out, err := readStdin(nil)
+	ctx := context.Background()
+
+	ctx = ContextWithStdin(ctx, strings.NewReader("foo"))
+	out, err := readStdin(ctx, nil)
 	assert.NoError(t, err)
 	assert.Equal(t, []byte("foo"), out)
 
-	stdin = errorReader{}
-	_, err = readStdin(nil)
+	ctx = ContextWithStdin(ctx, errorReader{})
+	_, err = readStdin(ctx, nil)
 	assert.Error(t, err)
 }
