@@ -8,7 +8,7 @@ import (
 )
 
 func checkExperimental(ctx context.Context) error {
-	if !config.FromContext(ctx).Experimental {
+	if !config.ExperimentalEnabled(ctx) {
 		return fmt.Errorf("experimental function, but experimental mode not enabled")
 	}
 	return nil
