@@ -1,7 +1,6 @@
 package config
 
 import (
-	"context"
 	"net/http"
 	"net/url"
 	"os"
@@ -14,6 +13,7 @@ import (
 
 	"github.com/hairyhenderson/gomplate/v3/internal/iohelpers"
 	"github.com/stretchr/testify/assert"
+	"gopkg.in/yaml.v3"
 )
 
 func TestParseConfigFile(t *testing.T) {
@@ -42,6 +42,11 @@ context:
   .:
     url: file:///data.json
 
+plugins:
+  foo:
+    cmd: echo
+    pipe: true
+
 pluginTimeout: 2s
 `
 	expected = &Config{
@@ -63,7 +68,10 @@ pluginTimeout: 2s
 				URL: mustURL("file:///data.json"),
 			},
 		},
-		OutMode:       "644",
+		OutMode: "644",
+		Plugins: map[string]PluginConfig{
+			"foo": {Cmd: "echo", Pipe: true},
+		},
 		PluginTimeout: 2 * time.Second,
 	}
 
@@ -298,43 +306,142 @@ func TestMergeFrom(t *testing.T) {
 	cfg = &Config{
 		Input:       "hello world",
 		OutputFiles: []string{"-"},
-		Plugins: map[string]string{
-			"sleep": "echo",
+		Plugins: map[string]PluginConfig{
+			"sleep": {Cmd: "echo"},
 		},
 		PluginTimeout: 500 * time.Microsecond,
 	}
 	other = &Config{
 		InputFiles:  []string{"-"},
 		OutputFiles: []string{"-"},
-		Plugins: map[string]string{
-			"sleep": "sleep.sh",
+		Plugins: map[string]PluginConfig{
+			"sleep": {Cmd: "sleep.sh"},
 		},
 	}
 	expected = &Config{
 		Input:       "hello world",
 		OutputFiles: []string{"-"},
-		Plugins: map[string]string{
-			"sleep": "sleep.sh",
+		Plugins: map[string]PluginConfig{
+			"sleep": {Cmd: "sleep.sh"},
 		},
 		PluginTimeout: 500 * time.Microsecond,
 	}
 
 	assert.EqualValues(t, expected, cfg.MergeFrom(other))
+
+	cfg = &Config{
+		Input:   "hello world",
+		OutMode: "644",
+	}
+	other = &Config{
+		OutputFiles: []string{"out.txt"},
+		Context: map[string]DataSource{
+			"foo": {
+				URL: mustURL("https://example.com/foo.yaml"),
+				Header: http.Header{
+					"Accept": {"application/json"},
+				},
+			},
+			"bar": {URL: mustURL("stdin:///")},
+		},
+		DataSources: map[string]DataSource{
+			"data": {
+				URL: mustURL("file:///data.json"),
+			},
+			"moredata": {
+				URL: mustURL("https://example.com/more.json"),
+				Header: http.Header{
+					"Authorization": {"Bearer abcd1234"},
+				},
+			},
+		},
+	}
+	expected = &Config{
+		Input:       "hello world",
+		OutputFiles: []string{"out.txt"},
+		Context: map[string]DataSource{
+			"foo": {
+				URL: mustURL("https://example.com/foo.yaml"),
+				Header: http.Header{
+					"Accept": {"application/json"},
+				},
+			},
+			"bar": {URL: mustURL("stdin:///")},
+		},
+		DataSources: map[string]DataSource{
+			"data": {
+				URL: mustURL("file:///data.json"),
+			},
+			"moredata": {
+				URL: mustURL("https://example.com/more.json"),
+				Header: http.Header{
+					"Authorization": {"Bearer abcd1234"},
+				},
+			},
+		},
+		OutMode: "644",
+	}
+
+	assert.EqualValues(t, expected, cfg.MergeFrom(other))
+
+	// test template merging & a few other things
+	cfg = &Config{
+		InputDir:    "indir/",
+		ExcludeGlob: []string{"*.txt"},
+		Templates: Templates{
+			"foo": {
+				URL: mustURL("file:///foo.yaml"),
+			},
+			"bar": {
+				URL:    mustURL("stdin:///"),
+				Header: http.Header{"Accept": {"application/json"}},
+			},
+		},
+	}
+	other = &Config{
+		ExcludeGlob: []string{"*.yaml"},
+		OutputMap:   "${ .in }.out",
+		OutMode:     "600",
+		LDelim:      "${",
+		RDelim:      "}",
+		Templates: Templates{
+			"foo": {URL: mustURL("https://example.com/foo.yaml")},
+			"baz": {URL: mustURL("vault:///baz")},
+		},
+	}
+	expected = &Config{
+		InputDir:    "indir/",
+		ExcludeGlob: []string{"*.yaml"},
+		OutputMap:   "${ .in }.out",
+		OutMode:     "600",
+		LDelim:      "${",
+		RDelim:      "}",
+		Templates: Templates{
+			"foo": {URL: mustURL("https://example.com/foo.yaml")},
+			"bar": {
+				URL:    mustURL("stdin:///"),
+				Header: http.Header{"Accept": {"application/json"}},
+			},
+			"baz": {URL: mustURL("vault:///baz")},
+		},
+	}
+
+	assert.EqualValues(t, expected, cfg.MergeFrom(other))
 }
 
 func TestParseDataSourceFlags(t *testing.T) {
 	t.Parallel()
 	cfg := &Config{}
-	err := cfg.ParseDataSourceFlags(nil, nil, nil)
+	err := cfg.ParseDataSourceFlags(nil, nil, nil, nil)
 	assert.NoError(t, err)
 	assert.EqualValues(t, &Config{}, cfg)
 
 	cfg = &Config{}
-	err = cfg.ParseDataSourceFlags([]string{"foo/bar/baz.json"}, nil, nil)
+	err = cfg.ParseDataSourceFlags([]string{"foo/bar/baz.json"}, nil, nil, nil)
 	assert.Error(t, err)
 
 	cfg = &Config{}
-	err = cfg.ParseDataSourceFlags([]string{"baz=foo/bar/baz.json"}, nil, nil)
+	err = cfg.ParseDataSourceFlags([]string{"baz=foo/bar/baz.json"}, nil, nil, nil)
 	assert.NoError(t, err)
 	expected := &Config{
 		DataSources: map[string]DataSource{
@@ -347,6 +454,7 @@ func TestParseDataSourceFlags(t *testing.T) {
 	err = cfg.ParseDataSourceFlags(
 		[]string{"baz=foo/bar/baz.json"},
 		nil,
+		nil,
 		[]string{"baz=Accept: application/json"})
 	assert.NoError(t, err)
 	assert.EqualValues(t, &Config{
@@ -364,6 +472,7 @@ func TestParseDataSourceFlags(t *testing.T) {
 	err = cfg.ParseDataSourceFlags(
 		[]string{"baz=foo/bar/baz.json"},
 		[]string{"foo=http://example.com"},
+		nil,
 		[]string{"foo=Accept: application/json",
 			"bar=Authorization: Basic xxxxx"})
 	assert.NoError(t, err)
@@ -383,6 +492,28 @@ func TestParseDataSourceFlags(t *testing.T) {
 			"bar": {"Authorization": {"Basic xxxxx"}},
 		},
 	}, cfg)
+
+	cfg = &Config{}
+	err = cfg.ParseDataSourceFlags(
+		nil,
+		nil,
+		[]string{"foo=http://example.com", "file.tmpl", "tmpldir/"},
+		[]string{"foo=Accept: application/json",
+			"bar=Authorization: Basic xxxxx"})
+	assert.NoError(t, err)
+	assert.EqualValues(t, &Config{
+		Templates: Templates{
+			"foo": {
+				URL:    mustURL("http://example.com"),
+				Header: http.Header{"Accept": {"application/json"}},
+			},
+			"file.tmpl": {URL: mustURL("file.tmpl")},
+			"tmpldir/":  {URL: mustURL("tmpldir/")},
+		},
+		ExtraHeaders: map[string]http.Header{
+			"bar": {"Authorization": {"Basic xxxxx"}},
+		},
+	}, cfg)
 }
 
 func TestParsePluginFlags(t *testing.T) {
@@ -394,7 +525,7 @@ func TestParsePluginFlags(t *testing.T) {
 	cfg = &Config{}
 	err = cfg.ParsePluginFlags([]string{"foo=bar"})
 	assert.NoError(t, err)
-	assert.EqualValues(t, &Config{Plugins: map[string]string{"foo": "bar"}}, cfg)
+	assert.EqualValues(t, &Config{Plugins: map[string]PluginConfig{"foo": {Cmd: "bar"}}}, cfg)
 }
 
 func TestConfigString(t *testing.T) {
@@ -415,7 +546,10 @@ pluginTimeout: 5s
 		RDelim:      "R",
 		Input:       "foo",
 		OutputFiles: []string{"-"},
-		Templates:   []string{"foo=foo.t", "bar=bar.t"},
+		Templates: Templates{
+			"foo": {URL: mustURL("https://www.example.com/foo.tmpl")},
+			"bar": {URL: mustURL("/tmp/bar.t")},
+		},
 	}
 	expected = `---
 in: foo
@@ -423,17 +557,22 @@ outputFiles: ['-']
 leftDelim: L
 rightDelim: R
 templates:
-  - foo=foo.t
-  - bar=bar.t
+  foo:
+    url: https://www.example.com/foo.tmpl
+  bar:
+    url: file:///tmp/bar.t
 `
-	assert.Equal(t, expected, c.String())
+	assert.YAMLEq(t, expected, c.String())
 
 	c = &Config{
 		LDelim:      "L",
 		RDelim:      "R",
 		Input:       "long input that should be truncated",
 		OutputFiles: []string{"-"},
-		Templates:   []string{"foo=foo.t", "bar=bar.t"},
+		Templates: Templates{
+			"foo": {URL: mustURL("https://www.example.com/foo.tmpl")},
+			"bar": {URL: mustURL("/tmp/bar.t")},
+		},
 	}
 	expected = `---
 in: long inp...
@@ -441,10 +580,12 @@ outputFiles: ['-']
 leftDelim: L
 rightDelim: R
 templates:
-  - foo=foo.t
-  - bar=bar.t
+  foo:
+    url: https://www.example.com/foo.tmpl
+  bar:
+    url: file:///tmp/bar.t
 `
-	assert.Equal(t, expected, c.String())
+	assert.YAMLEq(t, expected, c.String())
 
 	c = &Config{
 		InputDir:  "in/",
@@ -476,6 +617,24 @@ pluginTimeout: 500ms
 `
 
 	assert.Equal(t, expected, c.String())
+
+	c = &Config{
+		Plugins: map[string]PluginConfig{
+			"foo": {
+				Cmd:  "bar",
+				Pipe: true,
+			},
+		},
+	}
+	expected = `---
+plugins:
+  foo:
+    cmd: bar
+    timeout: 0s
+    pipe: true
+`
+
+	assert.Equal(t, expected, c.String())
 }
 
 func TestApplyDefaults(t *testing.T) {
@@ -667,9 +826,9 @@ func TestAbsFileURL(t *testing.T) {
 }
 
 func TestParseDatasourceArgNoAlias(t *testing.T) {
-	key, ds, err := parseDatasourceArg("foo.json")
+	alias, ds, err := parseDatasourceArg("foo.json")
 	assert.NoError(t, err)
-	assert.Equal(t, "foo", key)
+	assert.Equal(t, "foo", alias)
 	assert.Equal(t, "file", ds.URL.Scheme)
 
 	_, _, err = parseDatasourceArg("../foo.json")
@@ -680,79 +839,92 @@ func TestParseDatasourceArgNoAlias(t *testing.T) {
 }
 
 func TestParseDatasourceArgWithAlias(t *testing.T) {
-	key, ds, err := parseDatasourceArg("data=foo.json")
+	alias, ds, err := parseDatasourceArg("data=foo.json")
 	assert.NoError(t, err)
-	assert.Equal(t, "data", key)
+	assert.Equal(t, "data", alias)
 	assert.Equal(t, "file", ds.URL.Scheme)
 	assert.True(t, ds.URL.IsAbs())
 
-	key, ds, err = parseDatasourceArg("data=/otherdir/foo.json")
+	alias, ds, err = parseDatasourceArg("data=/otherdir/foo.json")
 	assert.NoError(t, err)
-	assert.Equal(t, "data", key)
+	assert.Equal(t, "data", alias)
 	assert.Equal(t, "file", ds.URL.Scheme)
 	assert.True(t, ds.URL.IsAbs())
 	assert.Equal(t, "/otherdir/foo.json", ds.URL.Path)
 
 	if runtime.GOOS == "windows" {
-		key, ds, err = parseDatasourceArg("data=foo.json")
+		alias, ds, err = parseDatasourceArg("data=foo.json")
 		assert.NoError(t, err)
-		assert.Equal(t, "data", key)
+		assert.Equal(t, "data", alias)
 		assert.Equal(t, "file", ds.URL.Scheme)
 		assert.True(t, ds.URL.IsAbs())
 		assert.Equalf(t, byte(':'), ds.URL.Path[1], "Path was %s", ds.URL.Path)
 
-		key, ds, err = parseDatasourceArg(`data=\otherdir\foo.json`)
+		alias, ds, err = parseDatasourceArg(`data=\otherdir\foo.json`)
 		assert.NoError(t, err)
-		assert.Equal(t, "data", key)
+		assert.Equal(t, "data", alias)
 		assert.Equal(t, "file", ds.URL.Scheme)
 		assert.True(t, ds.URL.IsAbs())
 		assert.Equal(t, `/otherdir/foo.json`, ds.URL.Path)
 
-		key, ds, err = parseDatasourceArg("data=C:\\windowsdir\\foo.json")
+		alias, ds, err = parseDatasourceArg("data=C:\\windowsdir\\foo.json")
 		assert.NoError(t, err)
-		assert.Equal(t, "data", key)
+		assert.Equal(t, "data", alias)
 		assert.Equal(t, "file", ds.URL.Scheme)
 		assert.True(t, ds.URL.IsAbs())
 		assert.Equal(t, "C:/windowsdir/foo.json", ds.URL.Path)
 
-		key, ds, err = parseDatasourceArg("data=\\\\somehost\\share\\foo.json")
+		alias, ds, err = parseDatasourceArg("data=\\\\somehost\\share\\foo.json")
 		assert.NoError(t, err)
-		assert.Equal(t, "data", key)
+		assert.Equal(t, "data", alias)
 		assert.Equal(t, "file", ds.URL.Scheme)
 		assert.Equal(t, "somehost", ds.URL.Host)
 		assert.True(t, ds.URL.IsAbs())
 		assert.Equal(t, "/share/foo.json", ds.URL.Path)
 	}
 
-	key, ds, err = parseDatasourceArg("data=sftp://example.com/blahblah/foo.json")
+	alias, ds, err = parseDatasourceArg("data=sftp://example.com/blahblah/foo.json")
 	assert.NoError(t, err)
-	assert.Equal(t, "data", key)
+	assert.Equal(t, "data", alias)
 	assert.Equal(t, "sftp", ds.URL.Scheme)
 	assert.True(t, ds.URL.IsAbs())
 	assert.Equal(t, "/blahblah/foo.json", ds.URL.Path)
 
-	key, ds, err = parseDatasourceArg("merged=merge:./foo.yaml|http://example.com/bar.json%3Ffoo=bar")
+	alias, ds, err = parseDatasourceArg("merged=merge:./foo.yaml|http://example.com/bar.json%3Ffoo=bar")
 	assert.NoError(t, err)
-	assert.Equal(t, "merged", key)
+	assert.Equal(t, "merged", alias)
 	assert.Equal(t, "merge", ds.URL.Scheme)
 	assert.Equal(t, "./foo.yaml|http://example.com/bar.json%3Ffoo=bar", ds.URL.Opaque)
 }
 
-func TestContextWithConfig(t *testing.T) {
-	ctx := context.Background()
-	cfg := &Config{}
-	ctx = ContextWithConfig(ctx, cfg)
-	assert.Equal(t, cfg, ctx.Value(cfgContextKey))
-}
+func TestPluginConfig_UnmarshalYAML(t *testing.T) {
+	in := `foo`
+	out := PluginConfig{}
+	err := yaml.Unmarshal([]byte(in), &out)
+	assert.NoError(t, err)
+	assert.EqualValues(t, PluginConfig{Cmd: "foo"}, out)
 
-func TestFromContext(t *testing.T) {
-	cfg := &Config{}
-	ctx := context.WithValue(context.Background(),
-		cfgContextKey, cfg)
-	assert.Equal(t, cfg, FromContext(ctx))
+	in = `[foo, bar]`
+	out = PluginConfig{}
+	err = yaml.Unmarshal([]byte(in), &out)
+	assert.Error(t, err)
 
-	ctx = context.Background()
-	cfg = FromContext(ctx)
-	// assert that the returned config looks like a default one
-	assert.Equal(t, "{{", cfg.LDelim)
+	in = `cmd: foo`
+	out = PluginConfig{}
+	err = yaml.Unmarshal([]byte(in), &out)
+	assert.NoError(t, err)
+	assert.EqualValues(t, PluginConfig{Cmd: "foo"}, out)
+
+	in = `cmd: foo
+timeout: 10ms
+pipe: true
+`
+	out = PluginConfig{}
+	err = yaml.Unmarshal([]byte(in), &out)
+	assert.NoError(t, err)
+	assert.EqualValues(t, PluginConfig{
+		Cmd:     "foo",
+		Timeout: time.Duration(10) * time.Millisecond,
+		Pipe:    true,
+	}, out)
 }
