@@ -17,8 +17,10 @@ func NormalizeFileMode(mode os.FileMode) os.FileMode {
 }
 
 func windowsFileMode(mode os.FileMode) os.FileMode {
-	// non-owner and execute bits are stripped
-	mode &^= 0o177
+	// non-owner and execute bits are stripped on files
+	if !mode.IsDir() {
+		mode &^= 0o177
+	}
 
 	if mode&0o200 != 0 {
 		// writeable implies read/write on Windows
